@@ -1,6 +1,7 @@
 use 5.014;
 use strict;
 use warnings;
+no warnings 'void';
 
 use Carp ();
 use Exporter::Tiny ();
@@ -13,7 +14,7 @@ use Sub::Name ();
 package Kavorka;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.013';
+our $VERSION   = '0.014';
 
 our @ISA         = qw( Exporter::Tiny );
 our @EXPORT      = qw( fun method );
@@ -63,33 +64,40 @@ sub _exporter_expand_sub
 	Module::Runtime::use_package_optimistically($implementation)->can('parse')
 		or Carp::croak("No suitable implementation for keyword '$name'");
 	
-	$^H{$me} .= "$name=$implementation ";
+	# Kavorka::Multi (for example) needs to know what Kavorka keywords are
+	# currently in scope.
+	$^H{'Kavorka'} .= "$name=$implementation ";
 	
-	no warnings 'void';
+	# This is the code that gets called at run-time.
+	#
 	my $code = Sub::Name::subname(
 		"$me\::$name",
 		sub {
-			$name; # close over name to prevent optimization
 			my $subroutine = shift;
+			$name; # close over name to prevent optimization
+			
+			# Post-parse clean-up
 			$subroutine->_post_parse();
+			
+			# Store $subroutine for introspection
 			$INFO{ $subroutine->body } = $subroutine;
 			
+			# Install sub
 			my @r = wantarray
 				? $subroutine->install_sub
 				: scalar($subroutine->install_sub);
 			
+			# Prevents a cycle between %INFO and $subroutine.
 			Scalar::Util::weaken($subroutine->{body})
 				unless Scalar::Util::isweak($subroutine->{body});
 			
-			my $closed_over = PadWalker::closed_over($subroutine->{body});
-			my $caller_vars = PadWalker::peek_my(1);
-			$closed_over->{$_} = $caller_vars->{$_} for keys %$closed_over;
-			PadWalker::set_closed_over($subroutine->{body}, $closed_over);
-			
 			wantarray ? @r : $r[0];
 		},
 	);
 	
+	# This joins up the code above with our custom parsing via
+	# Parse::Keyword
+	#
 	Parse::Keyword::install_keyword_handler(
 		$code => Sub::Name::subname(
 			"$me\::parse_$name",
@@ -104,6 +112,7 @@ sub _exporter_expand_sub
 		),
 	);
 	
+	# Symbol for Exporter::Tiny to export
 	return ($name => $code);
 }
 
